@@ -1,75 +1,66 @@
-from .download_selenium_driver import DriverType
-from typing import Optional, TYPE_CHECKING, Union
+from .download_selenium_driver import DriverType, download_driver
+from typing import Optional
 import os
+import tempfile
 import pathlib
 
-if TYPE_CHECKING:
-    from selenium import webdriver
 
-def selenium_driver_options(driver_type: DriverType) -> Union['webdriver.ChromeOptions', 'webdriver.FirefoxOptions']:
+def selenium_driver(driver_type: DriverType, version: Optional[str] = None) -> pathlib.Path:
     """
     Returns the selenium driver. 
     Args:
-        driver_type (DriverType): The type of selenium driver to load ('chrome' or 'firefox').
+        driver_type (DriverType): The type of driver to download ('chrome' or 'firefox').
+        version (str, optional): The version of the driver to download. 
+                                 If None, downloads the latest stable version. Defaults to None.
     
     Returns:
-        pathlib.Path: The selenium webdriver instance.
+        str: The full path to the downloaded driver executable.
     """
-    if os.environ.get('HOME') is None:
-        os.environ['HOME'] = '/tmp/home'
-        if not os.path.exists(os.environ['HOME']):
-            os.makedirs(os.environ['HOME'])
+    selenium_download_path_str = os.environ.get('DATALOG_SELENIUM_DRIVER_PATH', None)
+    if selenium_download_path_str is None:
+        # use temp directory
+        selenium_download_path_str = tempfile.gettempdir()
+    selenium_download_path = pathlib.Path(selenium_download_path_str)
+    
+    download_file = download_driver(driver_type, selenium_download_path, version)
     if driver_type == 'chrome':
         # make executable
+        download_file.chmod(download_file.stat().st_mode | 0o111)
+        
         from selenium import webdriver
+        from selenium.webdriver.chrome.service import Service as ChromeService
+        service = ChromeService(executable_path=str(download_file))
         options = webdriver.ChromeOptions()
         headless = os.environ.get('DATALOG_GUI', '0') == '0'
         if headless:
             options.add_argument('--headless=new')
         options.add_argument("--disable-gpu")
         options.add_argument("--no-sandbox")
-        return options
         
+        return webdriver.Chrome(service=service, options=options)
     elif driver_type == 'firefox':
+        # make executable
+        download_file.chmod(download_file.stat().st_mode | 0o111)
+        
         from selenium import webdriver
+        from selenium.webdriver.firefox.service import Service as GeckoService
+        service = GeckoService(executable_path=str(download_file))
         options = webdriver.FirefoxOptions()
-        options.binary_location="/usr/bin/firefox"
         headless = os.environ.get('DATALOG_GUI', '0') == '0'
         if headless:
             options.add_argument('--headless')
-        return options
-    raise ValueError(f"Unsupported driver type: {driver_type}")
-
-def selenium_driver(driver_type: DriverType) -> Union['webdriver.Chrome', 'webdriver.Firefox']:
-    """
-    Returns the selenium driver. 
-    Args:
-        driver_type (DriverType): The type of selenium driver to load ('chrome' or 'firefox').
-    
-    Returns:
-        pathlib.Path: The selenium webdriver instance.
-    """
-    options = selenium_driver_options(driver_type)
-    if driver_type == 'chrome':
-        driver = "/usr/bin/chromedriver"        
-        from selenium import webdriver
-        from selenium.webdriver.chrome.service import Service as ChromeService
-        service = ChromeService(executable_path=str(driver))
-        return webdriver.Chrome(service=service, options=options)
-    elif driver_type == 'firefox':
-        driver = "/usr/bin/geckodriver"
-        from selenium import webdriver
-        from selenium.webdriver.firefox.service import Service as GeckoService
-        service = GeckoService(executable_path=str(driver))
         return webdriver.Firefox(service=service, options=options)
-        
+    raise ValueError(f"Unsupported driver type: {driver_type}")
+    
+    
 class DatallogSeleniumDriver():
-    def __init__(self, driver_type: DriverType):
+    def __init__(self, driver_type: DriverType, version: Optional[str] = None):
         self.driver_type = driver_type
+        self.version = version
         self.driver: Optional[pathlib.Path] = None
 
     def __enter__(self):
-        self.driver = selenium_driver(self.driver_type)
+        self.driver = selenium_driver(self.driver_type, self.version)
         return self.driver
 
     def __exit__(self, exc_type, exc_value, traceback):
